#!/bin/env python
# -*- coding: UTF-8 -*-

import os
import signal
import time
import struct
import socket
import re
import list_cli_crv

# Clase para la creacion de objetos encargados de gestionar las comunicaciones entre los emisores y
# el receptor
# Los objetos de esta clase se encargan de gestionar las conexiones de los receptores para
# almacenar sus datos en un listado de receptores conocidos que se usa para la emision 
# automatica del CRV
class Recepcion_Clientes:
	# Funcion encargada de terminar las comunicaciones con los receptores
	def cerrar(self):
		self.sock.close()
		os.kill(self.pid,signal.SIGKILL)
	
	# Funcion para obtener la direccion MAC de un host a partir de su direccion IP
	# Parametros de entrada:
	# - ip: direccion IP del host que se desea obtener la MAC
	# Parametros de retorno:
	# - Devuelve la direccion MAC del host
	def obtener_dir_mac(self, ip):
		mac = ''
		fichero_tabla_arp = open('/proc/net/arp', 'r')
		tabla_arp = fichero_tabla_arp.readlines()
		for direccion in tabla_arp:
			datos_direccion = re.split('\s+', direccion)
			if datos_direccion[0] == ip:
				return datos_direccion[3]
		return mac
	
	# Funcion encargada de escribir los datos de un receptor en la lista de receptores conocidos
	# Parametros de entrada:
	# - mac: direccion mac del receptor
	# - ip: direccion ip del receptor
	def actualizar_datos_cliente(self, mac, ip):
		nuevo = 0
                fichero_lista = open('%slista_clientes.txt' % self.PATHLISTA, 'r')
                lista_clientes = fichero_lista.readlines()
                fichero_lista.close()
                fichero_lista = open('%slista_clientes.txt' % self.PATHLISTA, 'w')
                for elemento in lista_clientes: # buscar si el receptor se encuentra en la lista de receptores conocidos
                        datos_elemento = elemento.split(';;')
                        if mac == datos_elemento[0]: # El receptor ya existe y se actualiza solo la IP
                                datos_elemento[1] = ip
				nuevo = 1
                        elemento = '%s;;%s;;%s;;%s;;\n' % (datos_elemento[0], datos_elemento[1], datos_elemento[2], datos_elemento[3])
                        fichero_lista.write(elemento)
		if nuevo == 0: # Receptor nuevo 
                        elemento = '%s;;%s;;%s;;True;;\n' % (mac, ip, ' ')
                        fichero_lista.write(elemento)
			
                fichero_lista.close()

	# Constructor de la clase
	def __init__(self):
                self.PATHLISTA = '/usr/share/crv/'

		# Creacion de un socket de escucha en el puerto 50001
		self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
		self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
		self.sock.bind(('',50001))
		self.sock.listen(20)

		# Se bifurca la ejecucion de la aplicacion
		self.pid = os.fork()
		if self.pid == 0: # El proceso hijo se encarga de gestionar las conexiones de los receptores
			while 1:
				cliente, addr = self.sock.accept()
				pid_despachar = os.fork()
				if pid_despachar == 0:
					# se obtiene la informacion sensible de las conexiones entrantes y se actualiza la lista
					# de receptores conocidos
					mac = self.obtener_dir_mac(addr[0])
					self.actualizar_datos_cliente(mac, addr[0])
					cliente.close()
					exit(0)
				else:
					cliente.close()
					os.wait()

# Clase que gestiona los mensajes de terminacion de los receptores
# se encarga de esperar los mensajes de terminacion cuando se visualizan
# videos o audios
#class Cerrar_Cliente:
	# Funcion que cierra las comunicaciones
#	def terminar(self):
#		self.sock.close()

	# Funcion encargada de esperar el mensaje de terminacion del emisor
#	def esperar_cierre(self):
#		mensaje = ''
#		while mensaje != "opcionXX_000000":
#			cliente, addr = self.sock.accept()
#			if addr[0] == self.ip_emisor:
#				mensaje = cliente.recv(1024)
#			cliente.close()
#	# Constructor de la clase
#	# Parametros de entrada:
#	# -ip: direccion ip del emisor
#	def __init__(self, ip):
#		self.ip_emisor = ip
#		# Se crea el socket de escucha y se conecta en el puerto 50003
#		self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
#		self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
#		self.sock.bind(('',50003))
#		self.sock.listen(20)
#			

# Clase para el envio de mensajes a los receptores
class Emisor_Cliente:
	# Funcion que devuel True si la comiunicacion con el receptor se ha producido
	# correctamente o False en caso contrario
	#def resultado(self):
	#	print self.recepcion
	#	if self.recepcion == "OK":
	#		return True
	#	else:	
	#		return False
	
	# Funcion encargada de enviar un mensaje a un receptor
	# Parametros de entrada:
	# - ip: direccion ip del receptor
	# - mensaje: mensaje que se va a enviar
	def emitir(self, ip, mensaje):
		try: # Se intenta la conexion con el receptor
			self.sock.connect((ip,50002))
		except socket.error, msg: # El receptor no responde
			print "error de: %s" % msg
			#self.recepcion = 'NOOK'
			self.sock.close()
			self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
			self.sock.settimeout(10.0)
			return (1)

		# Se manda el mensaje
		self.sock.send(mensaje)
		try: # Se espera la respuesta del receptor
			self.recepcion = self.sock.recv(1024)
		except socket.error, msg: # Si el receptor no responde
			print "error de: %s" % msg
			#self.recepcion = 'NOOK'
			self.sock.close()
			self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
			self.sock.settimeout(10.0)
			return (1)
		#self.recepcion = 'OK'

                fichero_lista = open('%slista_clientes.txt' % self.PATHLISTA, 'r')
                lista_clientes = fichero_lista.readlines()
                fichero_lista.close()
                fichero_lista = open('%slista_clientes.txt' % self.PATHLISTA, 'w')
                for elemento in lista_clientes: # buscar si el receptor se encuentra en la lista de receptores conocidos
                        datos_elemento = elemento.split(';;')
                        if ip == datos_elemento[1]: # El receptor ya existe y se actualiza solo la IP
                        	elemento = '%s;;%s;;%s;;True;;\n' % (datos_elemento[0], datos_elemento[1], datos_elemento[2])
                        fichero_lista.write(elemento)
		fichero_lista.close()	

		self.sock.close()
		self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
		self.sock.settimeout(10.0)
	
	# Funcion encargada de enviar el mensaje de cerrar a un receptor
	# Parametros de entrada:
	# -ip: direccion ip del receptor
	def emitir_cerrar(self, ip):
		mensaje = 'opcionXX_000000\0'
		try: # Se intenta la conexion con el receptor
			self.sock.connect((ip,50002))
		except socket.error, msg: # El receptor no responde
			print "error de: %s" % msg
			return (1)
		
		# Se manda el mensaje
		self.sock.send(mensaje)
		self.sock.close()
		self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
		self.sock.settimeout(10.0)
	
	# Funcion encargada de terminar las comunicaciones con los receptores
	def terminar(self):
		self.sock.close()
		
	# Constructor de la clase encargado de crear el socket de conexion
	def __init__(self):
		#self.recepcion = ''
                self.PATHLISTA = '/usr/share/crv/'
		self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
		self.sock.settimeout(1.2)

# Clase para la comunicacion entre el receptor y el emisor
# Se encarga de conectar con un emisor para ser incluido en
# la lista de receptores conocidos del emisor
class Comunicador_Clientes:
	# Funcion encargada de conectar con el emisor
	def emitir(self, ip):
		try: # Se intenta la conexion con el emisor
			self.sock.connect((ip,50001))
		except socket.error, msg: # En caso de error
			print "error de: %s" % msg
			self.sock.close()
			return(1)
		self.sock.close()
		return(0)
		
	# Constructor de la clase encargado de crear el socket de conexion
	def __init__(self):
		self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
		self.sock.settimeout(1.2)
