#!/bin/env python
# -*- coding: UTF-8 -*-

import os
import re
import string
import grp
import signal
import time
import list_cli_crv
import comunicacion_crv
import mensaje_aviso_crv
import pygtk
pygtk.require('2.0')
import gtk

# Clase para la creacion de controles de emision de CRV de escritorio
# Se encarga de crear los controles necesarios para realizar la
# emision de video por el CRV
class Control_Crv_Video:
	# Funcion que oculta el bloque completo en la ventana principal
	def hide(self):
		self.video_frame.hide()

	# Funcion que muestra el boque completo en la ventana principal
	def show(self):
		self.video_frame.show()

	# Funcion para cancelar la seleccion de archivo
	def cancelar_seleccion(self, widget, data = None):
		self.sel_archivo.hide()

	
	# Funcion que escribe la ruta del archivo seleccionado en el cuadro de entrada de texto de archivo
	def fichero_seleccionado(self, widget, data = None):
		self.entrada_sel_archivo.set_text(self.sel_archivo.get_selections()[0])

	# Funcion que gestiona el selector del tipo de video que se va a emitir
	def fuente_video(self, widget, data=None):
		if data == "1":
			self.tipo_video = 'dvd'
		elif data == "2":
			self.tipo_video = 'vcd'
		elif data == "3":
			self.tipo_video = 'archivo'
			if widget.get_active() == True:
				self.et_sel_archivo.set_sensitive(True)	
				self.entrada_sel_archivo.set_sensitive(True)	
				self.boton_sel_archivo.set_sensitive(True)
			else:
				self.et_sel_archivo.set_sensitive(False)	
				self.entrada_sel_archivo.set_sensitive(False)	
				self.boton_sel_archivo.set_sensitive(False)
		else:
			print "OPCION INCORRECTA"

	
	# Funcion encargada de seleccionar un archivo del sistema de fichero
	def seleccionar_archivo(self, widget, data = None):
		# Se crea un cuadro de dialogo para la seleccion de archivo
		self.sel_archivo = gtk.FileSelection("Seleccionar archivo")
		self.sel_archivo.hide_fileop_buttons()
		self.sel_archivo.set_select_multiple(False)
		self.sel_archivo.connect("destroy", self.cancelar_seleccion)
		self.sel_archivo.ok_button.connect("clicked", self.fichero_seleccionado)
		self.sel_archivo.ok_button.connect("clicked", lambda w: self.sel_archivo.destroy())
		self.sel_archivo.cancel_button.connect("clicked", lambda w: self.sel_archivo.destroy())
		self.sel_archivo.set_filename('%s/' % self.home)
		self.sel_archivo.set_size_request(700,400)
		self.sel_archivo.show()

	# Funcion que gestiona el selector de calidad
	def seleccionar_calidad(self, widget, data = None):
		self.calidad = data

	# Funcion que gestiona el selector de formato
	def seleccionar_formato(self, widget, data = None):
		self.formato = data

	# Funcion que devuelve el tipo de video seleccionado para la emsion
	def get_tipo_video(self):
		return self.tipo_video

	# Funcion que devuelve la calidad seleccionada
	def get_calidad(self):
		return self.calidad

	# Funcion que devuelve el formato seleccionado
	def get_formato(self):
		return self.formato
	
	# Funcion que devuelve el archivo seleccionado para la emision
	def get_archivo(self):
		return self.entrada_sel_archivo.get_text()

	# Constructor de la clase encargado de generar el bloque de la interfaz para
	# el uso de la emision de video
	def __init__(self):
		self.tipo_video = ''
		self.calidad = ''
		self.formato = ''
		self.home = os.environ.get("HOME")

		# Se crea el contenedor para el bloque
		self.video_frame = gtk.Frame()
		self.video_frame.set_shadow_type(gtk.SHADOW_NONE)
		self.video_frame.set_size_request(390, 260)
		self.video_fixed = gtk.Fixed()
		self.video_frame.add(self.video_fixed)

		# Selector de fuente de reproduccion		
		self.et_sel_fuente_video = gtk.Label()
		self.et_sel_fuente_video.set_markup("<b>Seleccionar el tipo de vídeo para reproducir</b>")
		self.et_sel_fuente_video.set_use_markup(True)
		self.video_fixed.put(self.et_sel_fuente_video, 10, 0)
		self.selector = gtk.RadioButton(None, "Reproducir un DVD desde el lector de DVD")
		self.video_fixed.put(self.selector, 10, 20)
		self.tipo_video = 'dvd'
		self.selector.connect("toggled", self.fuente_video, "1")
		self.selector.show()
		self.selector = gtk.RadioButton(self.selector, "Reproducir un VCD desde el lector de CD/DVD")
		self.video_fixed.put(self.selector, 10, 40)
		self.selector.connect("toggled", self.fuente_video, "2")
		self.selector.show()
		self.selector = gtk.RadioButton(self.selector, "Reproducir un archivo de vídeo desde el disco duro")
		self.video_fixed.put(self.selector, 10, 60)
		self.selector.connect("toggled", self.fuente_video, "3")
		self.selector.show()
		
		# Selector de archivo de reproduccion
		self.et_sel_archivo = gtk.Label("Seleccionar el archivo que se quiere reproducir")
		self.video_fixed.put(self.et_sel_archivo, 10, 90)
		self.entrada_sel_archivo = gtk.Entry()
		self.entrada_sel_archivo.set_size_request(250, 28)
		self.video_fixed.put(self.entrada_sel_archivo, 10, 110)
		self.boton_sel_archivo = gtk.Button("Buscar")
		self.boton_sel_archivo.connect("clicked", self.seleccionar_archivo, None)
		self.boton_sel_archivo.set_size_request(100, 28)
		self.video_fixed.put(self.boton_sel_archivo, 270, 110)
		self.et_sel_archivo.set_sensitive(False)
		self.entrada_sel_archivo.set_sensitive(False)
		self.boton_sel_archivo.set_sensitive(False)
		self.et_sel_archivo.show()
		self.entrada_sel_archivo.show()
		self.boton_sel_archivo.show()

		# Selector de calidad de emision
		self.et_sel_calidad = gtk.Label()
		self.et_sel_calidad.set_markup("<b>Seleccionar calidad</b>")
		self.et_sel_calidad.set_use_markup(True)
		self.video_fixed.put(self.et_sel_calidad, 10, 150)
		self.et_sel_calidad.show()
		self.selector_calidad = gtk.RadioButton(None, "Calidad baja")
		self.selector_calidad.connect("toggled", self.seleccionar_calidad, "baja")
		self.calidad = "baja"
		self.video_fixed.put(self.selector_calidad, 10, 170)
		self.selector_calidad.show()
		self.selector_calidad = gtk.RadioButton(self.selector_calidad, "Calidad media")
		self.selector_calidad.connect("toggled", self.seleccionar_calidad, "media")
		self.video_fixed.put(self.selector_calidad, 10, 190)
		self.selector_calidad.show()
		self.selector_calidad = gtk.RadioButton(self.selector_calidad, "Calidad alta")
		self.selector_calidad.connect("toggled", self.seleccionar_calidad, "alta")
		self.video_fixed.put(self.selector_calidad, 10, 210)
		self.selector_calidad.show()
		self.selector_calidad = gtk.RadioButton(self.selector_calidad, "Calidad real")
		self.selector_calidad.connect("toggled", self.seleccionar_calidad, "real")
		self.video_fixed.put(self.selector_calidad, 10, 230)
		self.selector_calidad.show()
	
		# Selector de formato de imagen de emision
		self.et_sel_formato = gtk.Label()
		self.et_sel_formato.set_markup("<b>Seleccionar formato</b>")
		self.et_sel_formato.set_use_markup(True)
		self.video_fixed.put(self.et_sel_formato, 200, 150)
		self.et_sel_formato.show()
		self.selector_formato = gtk.RadioButton(None, "Normal 4:3")
		self.selector_formato.connect("toggled", self.seleccionar_formato, "4:3")
		self.formato = "4:3"
		self.video_fixed.put(self.selector_formato, 200, 170)
		self.selector_formato.show()
		self.selector_formato = gtk.RadioButton(self.selector_formato, "Panorámico 16:9")
		self.selector_formato.connect("toggled", self.seleccionar_formato, "16:9")
		self.video_fixed.put(self.selector_formato, 200, 190)
		self.selector_formato.show()
		self.selector_formato = gtk.RadioButton(self.selector_formato, "Cinemascope 235:100")
		self.selector_formato.connect("toggled", self.seleccionar_formato, "235:100")
		self.video_fixed.put(self.selector_formato, 200, 210)
		self.selector_formato.show()
		self.selector_formato = gtk.RadioButton(self.selector_formato, "Formato original")
		self.selector_formato.connect("toggled", self.seleccionar_formato, "0:0")
		self.video_fixed.put(self.selector_formato, 200, 230)
		self.selector_formato.show()
	
		# Se dibuja el bloque
		self.et_sel_fuente_video.show()
		self.video_fixed.show()

# Clase para la creacion de controles de emision de CRV de escritorio
# Se encarga de crear los controles necesarios para realizar la
# emision del escritorio por el CRV
class Control_Crv_Escritorio:
	# Funcion que oculta el bloque completo en la ventana principal
	def hide(self):
		self.escritorio_frame.hide()

	# Funcion que muestra el bloque completo en la ventana principal
	def show(self):
		self.escritorio_frame.show()
	
	# Funcion que devuelve la calidad de emision
	def get_calidad(self):
		return self.calidad

	# Funcion que devuelve si la opcion de bloqueo esta activada o desactivada
	def get_bloqueo(self):
		return self.sel_bloq.get_active()

	# Funcion encargada de gestionar el selector de calidad del bloque
	def seleccionar_calidad(self, widget, data = None):
		if data == "1":
			self.calidad = 'baja'
		elif data == "2":
			self.calidad = 'alta'
		else:
			print "OPCION INCORRECTA"
	
	# Constructor de la clase encargado de generar el bloque de la interfaz para
	# el uso de la emision de escritorio
	def __init__(self):
		self.calidad = ''
	
		# Se crea el contenedor del bloque
		self.escritorio_frame = gtk.Frame()
		self.escritorio_frame.set_shadow_type(gtk.SHADOW_NONE)
		self.escritorio_frame.set_size_request(390, 260)
		self.escritorio_fixed = gtk.Fixed()
		self.escritorio_frame.add(self.escritorio_fixed)
		
		# Se crea el selector de la calidad de emision del escritorio
		self.et_sel_cal = gtk.Label()
		self.et_sel_cal.set_markup("<b>Seleccionar la calidad de emisión del escritorio</b>")
		self.et_sel_cal.set_use_markup(True)
		self.escritorio_fixed.put(self.et_sel_cal, 10, 0)
		self.selector = gtk.RadioButton(None, "Calidad baja (recomendado para WiFi)")
		self.selector.connect("toggled", self.seleccionar_calidad, "1")
		self.escritorio_fixed.put(self.selector, 10, 20)
		self.calidad = 'baja'
		self.selector.show()
		self.selector = gtk.RadioButton(self.selector, "Calidad alta")
		self.selector.connect("toggled", self.seleccionar_calidad, "2")
		self.escritorio_fixed.put(self.selector, 10, 40)
		self.selector.show()

		# Se crea el widget para activar y desactivar el bloqueo de las pantallas de los receptores
		self.sel_bloq = gtk.CheckButton("Bloquear la pantalla de los alumnos durante la emisión")
		self.escritorio_fixed.put(self.sel_bloq, 5, 80)
		
		# Se dibuja el bloque en la interfaz de usuario
		self.et_sel_cal.show()
		self.sel_bloq.show()	
		self.escritorio_fixed.show()

# Clase para la creacion de controles de emision de CRV de audio
# Se encarga de crear los controles necesarios para realizar la
# emision de audio por el CRV
class Control_Crv_Audio:
	# Funcion para ocultar el bloque completo en la interfaz de usuario
	def hide(self):
		self.audio_frame.hide()

	# Funcion para mostrar el bloque completo en la interfaz de usuario
	def show(self):
		self.audio_frame.show()

	# Funcion encargada de gestionar el selector del tipo de audio
	def fuente_audio(self, widget, data=None):
		if data == "1": # Se selecciona la emision desde CD
			self.tipo_audio = "cd"
		elif data == "2": # Se selecciona la emision desde archivo
			self.tipo_audio = "archivo"
			if widget.get_active () == True:
				self.et_sel_archivo.set_sensitive(True)
				self.entrada_sel_archivo.set_sensitive(True)
				self.boton_sel_archivo.set_sensitive(True)
			else:
				self.et_sel_archivo.set_sensitive(False)
				self.entrada_sel_archivo.set_sensitive(False)
				self.boton_sel_archivo.set_sensitive(False)
		
	# Funcion encargada de seleccionar un archivo del sistema de fichero
	def seleccionar_archivo(self, widget, data = None):
		# Se crea un cuadro de dialogo para la seleccion de archivo
		self.sel_archivo = gtk.FileSelection("Seleccionar archivo")
		self.sel_archivo.hide_fileop_buttons()
		self.sel_archivo.set_select_multiple(False)
		self.sel_archivo.connect("destroy", self.cancelar_seleccion)
		self.sel_archivo.ok_button.connect("clicked", self.fichero_seleccionado)
		self.sel_archivo.ok_button.connect("clicked", lambda w: self.sel_archivo.destroy())
		self.sel_archivo.cancel_button.connect("clicked", lambda w: self.sel_archivo.destroy())
		self.sel_archivo.set_filename('%s/' % self.home)
		self.sel_archivo.set_size_request(700,400)
		self.sel_archivo.show()

	# Funcion que cancela la seleccion de archivo
	def cancelar_seleccion(self, widget, data = None):
		self.sel_archivo.hide()

	# Funcion que escribe la ruta del archivo seleccionado en el cuadro de entrada de texto de archivo
	def fichero_seleccionado(self, widget, data = None):
		self.entrada_sel_archivo.set_text(self.sel_archivo.get_selections()[0])

	# Funcion que devuelve el tipo de emision de audio que se va a realizar (cd o archivo)
	def get_tipo_audio(self):
		return self.tipo_audio

	# Funcion que devuelve el archivo para reproducir que se haya seleccionado
	def get_archivo(self):
		return self.entrada_sel_archivo.get_text()

	# Constructor de la clase encargado de generar el bloque de la interfaz para
	# el uso de la emision de audio
	def __init__(self):
		self.tipo_audio = ''
		self.home = os.environ.get("HOME")

		# Se crea el contenedor del bloque
		self.audio_frame = gtk.Frame()
		self.audio_frame.set_shadow_type(gtk.SHADOW_NONE)
		self.audio_frame.set_size_request(390, 260)
		self.audio_fixed = gtk.Fixed()
		self.audio_frame.add(self.audio_fixed)
		
		#Selector de fuente de emision de audio
		self.et_sel_fuente_audio = gtk.Label()
		self.et_sel_fuente_audio.set_markup("<b>Seleccionar el tipo de audio que se quiere  \nreproducir</b>")
		self.et_sel_fuente_audio.set_use_markup(True)
		self.audio_fixed.put(self.et_sel_fuente_audio, 10, 0)
		self.selector = gtk.RadioButton(None, "Reproducir un disco de audio desde el lector")
		self.tipo_audio = 'cd'
		self.audio_fixed.put(self.selector, 10, 40)
		self.selector.connect("toggled", self.fuente_audio, "1")
		self.selector.show()
		self.selector = gtk.RadioButton(self.selector, "Reproducir un archivo de música desde el disco \nduro local")
		self.audio_fixed.put(self.selector, 10, 60)
		self.selector.connect("toggled", self.fuente_audio, "2")
		
		#Buscar el archivo para reproducir
		self.et_sel_archivo = gtk.Label("Seleccionar el archivo que se quiere reproducir")
		self.audio_fixed.put(self.et_sel_archivo, 10, 120)
		self.entrada_sel_archivo = gtk.Entry()
		self.entrada_sel_archivo.set_size_request(250, 28)
		self.audio_fixed.put(self.entrada_sel_archivo, 10, 150)
		self.boton_sel_archivo = gtk.Button("Buscar")
		self.boton_sel_archivo.connect("clicked", self.seleccionar_archivo, None)
		self.boton_sel_archivo.set_size_request(100, 28)
		self.audio_fixed.put(self.boton_sel_archivo, 270, 150)

		# Se dibuja el bloque en la interfaz de la aplicacion
		self.et_sel_archivo.set_sensitive(False)
		self.entrada_sel_archivo.set_sensitive(False)
		self.boton_sel_archivo.set_sensitive(False)
		self.et_sel_archivo.show()
		self.entrada_sel_archivo.show()
		self.boton_sel_archivo.show()
		self.selector.show()
		self.et_sel_fuente_audio.show()
		self.audio_fixed.show()

# Clase encargada para la gestion de la red dentro del programa
# Dependiendo de la emision seleccionada o del estado de emision
# se mostraran unos datos u otros
class Gestor_Redes:
	# Funcion para mostrar el bloque completo
	def show(self):
		self.interfaz_frame.show()

	# Funcion para ocultar el bloque completo
	def hide(self):
		self.interfaz_frame.hide()
	
	# Funcion que determina si una interfaz de red es WiFi o no
	# Paramentros de entrada:
	# - interfaz: interfaz de red que se desea consultar
	# Paramentros de salida:
	# - True: si la interfaz es wifi
	# - False: si la interfaz no es wifi
	def es_wifi(self, interfaz):
		bloque_iwconfig_buffer = os.popen('/sbin/iwconfig %s' % interfaz,'r')
		bloque_iwconfig = bloque_iwconfig_buffer.read()
		pattern_wifi = re.compile('(^\w+)\s+(?!no wirelless extensions)')
		if re.search(pattern_wifi, bloque_iwconfig) == None:
			return False
		else:		
			return True

	# Funcuin que muestra en el bloque la informacion correspondiente cuando se selecciona
	# la opcion de usar el CRV como receptor
	def mostrar_en_crv_alumno(self):
		self.et_interfaz.hide()
		self.et_interfaz_emision.hide()
		self.boton_cambiar.hide()
		self.et_interfaz_alumno.show()
		
	# Funcion que muestra en el bloque la informacion correspondiente cuando se usa
	# la aplicacion como profesor para emitir antes de comenzar la emision
	def mostrar_en_crv_profesor(self):
		self.et_interfaz_alumno.hide()
		self.et_interfaz_emision.hide()
		self.et_interfaz.show()
		self.boton_cambiar.show()

	# Funcion que muestra en el bloque la informacion que se debe ver cuando se esta emitiendo
	def mostrar_en_emision(self):
		self.et_interfaz.hide()
		self.et_interfaz_alumno.hide()
		self.boton_cambiar.hide()
		self.et_interfaz_emision.show()

	# Funcion que deshabilita el uso del bloque completo
	def set_sensitive(self, estado):
		if estado == True:
			self.interfaz_frame.set_sensitive(True)
		elif estado == False:
			self.interfaz_frame.set_sensitive(False)
		else:
			print "Opcion incorrecta"

	# False: se buscan las interfaces cableadas
	# Funcion para realizar la busqueda de todas las interfaces de red que se encuentren
	# presente en el sistema
	# Paramentros de entrada:
	# - wifi: parametro booleano que indica si se buscan o no las interfaces wifi
	#		True: se buscan las interfaces wifi
	# 		False: se buscan las interfaces cableadas
	#		None: se buscan los dos tipos
	def buscar_interfaz_red(self, wifi = None):
		# Obtener todas las interfaces de red
		bloque_ifconfig_buffer = os.popen('/sbin/ifconfig', 'r')
		bloque_ifconfig = bloque_ifconfig_buffer.read()
		pattern_interfaces = re.compile('(\w+)\s+Link')
		interfaces = re.findall(pattern_interfaces, bloque_ifconfig)
		for interfaz in interfaces:
			if interfaz != 'lo': # descartar la interfaz de loopback
				bloque_ifconfig_solo_buffer = os.popen('/sbin/ifconfig %s' % interfaz,'r')
				bloque_ifconfig_solo = bloque_ifconfig_solo_buffer.read()
				pattern_bloqip = re.compile('addr:(\d+.\d+.\d+.\d+)')
				if re.search(pattern_bloqip, bloque_ifconfig_solo) != None: # comprobar que la interfaz tiene direccion IP asignada
					bloqip = re.findall(pattern_bloqip, bloque_ifconfig_solo)
					pattern_ip = re.compile('\d+.\d+.\d+.\d+')
					ip = re.findall(pattern_ip,bloqip[0])
					if self.es_wifi(interfaz) == wifi: # Si la interfaz coincide con lo que se busca (wifi o cable) se selecciona esa interfaz
						self.ip = ip[0]
						self.interfaz = interfaz
						break
					elif wifi == None: # Si la busqueda se realiza completa, esto se realiza para generar la lista completa de interfaces
						if self.es_wifi(interfaz) == True:
							self.imagen_tipo = gtk.gdk.pixbuf_new_from_file('%simg/red_wifi.png' % self.PATH)
							tipo = "Red de WiFi"
						else:
							self.imagen_tipo = gtk.gdk.pixbuf_new_from_file('%simg/red_cable.png' % self.PATH)
							tipo = "Red de cable"
						self.listado_ip_disp.append([self.imagen_tipo, ip[0], '%s - %s' % (interfaz, tipo)])
					else:
						print "PARAMENTROS INCORRECTOS"
	
	def delete_event(self, widget, data = None):
		widget.hide()
		
	def destroy(self, widget, data = None):
		widget.hide()
	
	# Funcion asociada al boton de Aceptar de la ventana para aplicar el cambio de interfaz
	def aceptar_cambio_ip(self, widget, data = None):
		# Se extra de la lista el elemento seleccionado
		(model, iter) = self.seleccion_vista_cambio_ip.get_selected()
		if iter != None: # Se ha seleccionado un elemento
			# Se realiza el cambio de interfaz
			icono = model.get_value(iter, 0)
			ip = model.get_value(iter, 1)
			tipo = model.get_value(iter, 2)
			self.imagen_interfaz.set_from_pixbuf(icono)
			self.et_ip.set_text('%s (%s)' % (ip, tipo))
			self.ip = ip
			
		self.window_cambio_ip.hide()

	# Funcion para cambiar la interfaz de red de emision
	# Se crea una ventana independiente con un listado de las interfaces de red
	# encontradas y con direccion IP asignada
	def cambiar_interfaz_red(self, widget, data = None):
		# Se crea la ventana donde se muestra la lista
		self.window_cambio_ip = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.window_cambio_ip.set_keep_above(True)
		self.window_cambio_ip.set_border_width(10)
		self.window_cambio_ip.set_icon_from_file("%simg/crv_icon.png" % self.PATH)
		self.window_cambio_ip.set_size_request(400, 240)
		self.window_cambio_ip.set_title("CRV - Cambiar IP")
		self.window_cambio_ip.set_resizable(False)
		self.window_cambio_ip.connect("delete_event", self.delete_event)
		self.window_cambio_ip.connect("destroy", self.destroy)
		
		# Se crea el contendor de la lista
		self.listado_interfaces_fixed = gtk.Fixed()
		self.listado_interfaces_fixed.set_size_request(380, 240)
		self.et_lista_cambiar_ip = gtk.Label()
		self.et_lista_cambiar_ip.set_markup("<b>Seleccionar la IP por la que desea emitir</b>")
		self.et_lista_cambiar_ip.set_use_markup(True)
		self.listado_interfaces_fixed.put(self.et_lista_cambiar_ip, 0, 0)
		self.et_lista_cambiar_ip.show()
		self.contenedor_lista_cambio_ip = gtk.ScrolledWindow()
		self.contenedor_lista_cambio_ip.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		self.contenedor_lista_cambio_ip.set_shadow_type(gtk.SHADOW_IN)

		# Crear lista, buscar interfaces y publicarlas en la lista
		self.listado_ip_disp = gtk.ListStore(gtk.gdk.Pixbuf, str, str)
		self.listado_ip_disp.clear()
		self.buscar_interfaz_red()
		self.vista_arbol_lista_ip_disp = gtk.TreeView()
		self.vista_arbol_lista_ip_disp.set_model(self.listado_ip_disp)

		# Elementos que componen la lista
		# Columna tipo de conexion icono
		self.contenido_columna_tipo_con = gtk.CellRendererPixbuf()
		self.columna_tipo_con = gtk.TreeViewColumn('Tipo', self.contenido_columna_tipo_con, pixbuf=0)
		self.vista_arbol_lista_ip_disp.append_column(self.columna_tipo_con)

		# Columna de direccion ip de la lista
		self.contenido_columna_dirip = gtk.CellRendererText()
		self.columna_dirip = gtk.TreeViewColumn('Dirección IP', self.contenido_columna_dirip, text=1)
		self.vista_arbol_lista_ip_disp.append_column(self.columna_dirip)

		# Columna de informacion adicional de la interfaz
		self.contenido_columna_dirip_info = gtk.CellRendererText()
		self.columna_dirip_info = gtk.TreeViewColumn('Informacion', self.contenido_columna_dirip_info, text=2)
		self.vista_arbol_lista_ip_disp.append_column(self.columna_dirip_info)

		# Configuracion de opciones de visibilidad de la lista
		self.seleccion_vista_cambio_ip = self.vista_arbol_lista_ip_disp.get_selection()
		self.seleccion_vista_cambio_ip.set_mode(gtk.SELECTION_SINGLE)

		# Dibujara la lista en la ventana
		self.listado_interfaces_fixed.put(self.contenedor_lista_cambio_ip, 0, 20)
		self.contenedor_lista_cambio_ip.add(self.vista_arbol_lista_ip_disp)
		self.contenedor_lista_cambio_ip.set_size_request(380, 160)
		self.contenedor_lista_cambio_ip.show_all()

		# Boton para aceptar el cambio de interfaz
		boton_aceptar_cambio_ip = gtk.Button("Aceptar")
		boton_aceptar_cambio_ip.connect("clicked", self.aceptar_cambio_ip, None)
		self.listado_interfaces_fixed.put(boton_aceptar_cambio_ip, 320, 190)
		
		# Dibujar la ventana
		self.window_cambio_ip.add(self.listado_interfaces_fixed)
		self.window_cambio_ip.show_all()
	
	# Funcion que devuelve la direccion IP utilizada para emitir	
	def get_ip(self):
		return self.ip
		
	# Contructor de la clase
	# Crea todos los elementos mostrados en la interfaz de usuario
	def __init__(self):
		# Inicializacion de paramentros
		self.PATH = '/usr/local/crv/'
		self.ip = ''
		self.interfaz = ''
		self.interfaz_frame = gtk.Frame()
		self.interfaz_frame.set_size_request(410, 80)
		self.interfaz_fixed = gtk.Fixed()
		
		# Crea los elementos del bloque
		self.et_interfaz = gtk.Label()
		self.et_interfaz.set_markup("<b>La siguiente dirección IP será utilizada para emitir</b>")
		self.et_interfaz.set_use_markup(True)
		self.interfaz_fixed.put(self.et_interfaz, 10, 5)
	
		self.et_interfaz_alumno = gtk.Label()
		self.et_interfaz_alumno.set_markup("<b>La dirección IP de este ordenador es la siguiente</b>")
		self.et_interfaz_alumno.set_use_markup(True)
		self.interfaz_fixed.put(self.et_interfaz_alumno, 10, 5)

		self.et_interfaz_emision = gtk.Label()
		self.et_interfaz_emision.set_markup("<b>Se está emitiendo por la siguiente dirección IP </b>")
		self.et_interfaz_emision.set_use_markup(True)
		self.interfaz_fixed.put(self.et_interfaz_emision, 10, 5)

		#busqueda de interfaces de red con prioridad para las cableadas
		self.buscar_interfaz_red(False)
		if self.ip == '' or self.interfaz == '':
			self.buscar_interfaz_red(True)
			if self.ip == '' or self.interfaz == '':
				tipo_red = "no disp."
				icono_red = "red_nodisponible.png"
				self.ip = "IP no disponible"
				self.interfaz = "---"
			else:
				tipo_red = "WiFi"
				icono_red = "red_wifi.png"
		else:
			tipo_red = "de cable"
			icono_red = "red_cable.png"
	
		# Se mumestra la interfaz de red seleccionada por defecto por la aplicacion
		self.imagen_interfaz = gtk.Image()
		self.imagen_interfaz.set_from_file('%simg/%s' % (self.PATH, icono_red))
		self.et_ip = gtk.Label('%s (%s - Red %s)' % (self.ip, self.interfaz, tipo_red))

		# Se ofrece la posibilidad de seleccionar otra interfaz de red disponible
		self.boton_cambiar = gtk.Button('Cambiar')
		self.boton_cambiar.connect("clicked", self.cambiar_interfaz_red, None)
		self.interfaz_fixed.put(self.imagen_interfaz, 25, 30)
		self.interfaz_fixed.put(self.et_ip, 60, 35)
		self.interfaz_fixed.put(self.boton_cambiar, 320, 30)
		self.boton_cambiar.show()
		self.imagen_interfaz.show()
		self.et_ip.show()

		# Se dibuja el bloque completo en la interfaz de usuario	
		self.interfaz_frame.add(self.interfaz_fixed)
		self.interfaz_fixed.show()
		self.et_interfaz.show()

# Clase para la creacion del icono de informacion del araea de notificaciones
class Notificacion_Estado:
	# Funcion que muestra la informacion referente aa la emision de escritorio
	# Paramentros de entrada:
	# - calidad: calidad de emision del escritorio
	# - bloque: indicador de si se ha bloqueado la pantalla de los alumnos o no se ha bloqueado
	def mensaje_escritorio(self, calidad, bloqueo):
		if bloqueo == True:
			cad_bloqueo = "La pantalla del alumnado está bloqueada durante la emisión"
		else:
			cad_bloqueo = "La pantalla del alumnado no está bloqueada durante la emisión"
		self.icono_notificacion.set_tooltip('Tipo de emisión:  escritorio\nCalidad de emisión:  %s\n%s' % (calidad, cad_bloqueo))

	# Funcion que muestra la informacion referente a la emision de video
	# Paramentros de entrada:
	# - tipo: tipo de video que se emite
	# - archivo: ruta hasta el archivo que se esta emitiendo
	# - calidad: calidad de emision
	# - formato: formato de la emision
	def mensaje_video(self, tipo, archivo, calidad, formato):
		if tipo == "dvd":
			cad_tipo = "Tipo de vídeo:  DVD"
		elif tipo == "vcd":
			cad_tipo = "Tipo de vídeo:  VCD"
		elif tipo == "archivo":
			cad_tipo = "Tipo de vídeo:  Archivo de video"
		else:
			print "OPCION INCORRECTA"
		
		if formato == "4:3":
			cad_formato = "Formato:  Normal (4:3)"
		elif formato == "16:9":
			cad_formato = "Formato:  Panorámico (16:9)"
		elif formato == "235:100":
			cad_formato = "Formato:  Cinemascope (235:100)"
		elif formato == "0:0":
			cad_formato = "Formato:  Normal"
		else:
			print "OPCION INCORRECTA"

		if tipo == "archivo":
			self.icono_notificacion.set_tooltip('Tipo de emisión:  vídeo\n%s\nArchivo en reproducción:  %s\nCalidad:  %s\n%s' % (cad_tipo, archivo, calidad, cad_formato))
		else:	
			self.icono_notificacion.set_tooltip('Tipo de emisión:  vídeo\n%s\nCalidad:  %s\n%s' % (cad_tipo, calidad, cad_formato))

	# Funcion que muestra la informacion referente a la emision de audio
	# Parametros de entrada:
	# - tipo: tipo de emision de audio
	# - archivo:  ruta al archivo de emision
	def mensaje_audio(self, tipo, archivo):
		if tipo == "cd":
			self.icono_notificacion.set_tooltip('Tipo de emisión:  audio\nTipo de audio:  CD')
		else:
			self.icono_notificacion.set_tooltip('Tipo de emisión:  audio\nTipo de audio:  Archivo de audio\nArchivo en reproducción:  %s' % archivo)

	# Funcion que muestra el icono en el area de notificacion
	def show(self):
		self.icono_notificacion.set_visible(True)

	# Funcion que oculta el icono del area de notificacion
	def hide(self):
		self.icono_notificacion.set_visible(False)	
	
	# Constructor de la clase encargada de crear el widget		
	def __init__(self):
		self.PATH = '/usr/local/crv/'
		self.icono_notificacion = gtk.StatusIcon()
		self.icono_notificacion.set_from_file('%simg/crv_icon.png' % self.PATH)
		self.icono_notificacion.set_visible(False)

# Clase para mostrar la interfaz del CRV Profesor (Emision)
# Gestiona el funcionamiento de las emisiones controlando:
# - El tipo de emision
# - Las opciones de cada tipo de emision
# - Seleccion de la interfaz de red de emision
# - Gestion de lista de los receptores conocidos
class crv_Profesor:
	# Funcion asociada al selector del tipo de emison y se encarga de modificar la
	# interfaz de usuario y dependiendo del tipo de seleccion mostrar las opciones
	# correspondiente a la emision elegida
	def selector_emision(self, widget, data=None):
		# Dependiendo de la opcion del selector se mostrara unos elementos u otros
		if data == "1": # Activar el emisor de escritorio
			if widget.get_active() == True:
				self.emisor_activo="escritorio"
				self.control_crv_escritorio.show()
				self.opciones_frame.set_label("Opciones de emisión de escritorio")
				self.tipo_emision = "escritorio"
			else:
				self.control_crv_escritorio.hide()
		elif data == "2": # Activar el emisor de video
			if widget.get_active() == True:
				self.emisor_activo="video"
				self.control_crv_video.show()
				self.opciones_frame.set_label("Opciones de emisión de vídeo")
				self.tipo_emision="video"
			else:
				self.control_crv_video.hide()
		elif data == "3": # Activar el emisor de audio
			if widget.get_active() == True:
				self.emisor_activo="audio"
				self.control_crv_audio.show()
				self.opciones_frame.set_label("Opciones de emisión de audio")
				self.tipo_emision="audio"
			else:
				self.control_crv_audio.hide()
		elif data == "4": # Activar el emisor como alumno
			if widget.get_active() == True:
				self.opciones_frame.set_label("Opciones de uso como alumno")
				self.crv_alumnos.show()
				self.botemi_frame.set_sensitive(False)
				self.clientes.listado_fixed.set_sensitive(False)
				self.crv_alumnos.cancelar_conexion.hide()
				self.interfaz_emision.mostrar_en_crv_alumno()
			else:
				self.crv_alumnos.hide()
				self.botemi_frame.set_sensitive(True)
				self.clientes.listado_fixed.set_sensitive(True)
				self.crv_alumnos.cancelar_conexion.show()
				self.interfaz_emision.mostrar_en_crv_profesor()
		else:
			print "Opcion incorrecta"

	# Funcion que prepara adecuadamente y ejecuta VNC para la emision de escritorio
	# Paramentros de entrada:
	# - calidad: calidad con la que se desea emitir (alta o baja)
	def ejecutar_emision_escritorio(self, calidad):
		# Dependiendo de la calidad seleccionada se ejecuta VNC con unos parametros u otro
		if calidad == 'alta':
			self.pid_proceso_emision_vnc = os.spawnlp(os.P_NOWAIT, '/usr/bin/x11vnc', '/usr/bin/x11vnc', '-viewonly', '-many', '-shared', '-rfbport', '50000')
		elif calidad == 'baja':
			self.pid_proceso_emision_vnc = os.spawnlp(os.P_NOWAIT, '/usr/bin/x11vnc', 'usr/bin/x11vnc', '-viewonly', '-many', '-shared', '-notruecolor', '-speeds', '6,2500,1', '-wait', '200', '-onetile', '-solid', 'darkblue', '-rfbport', '50000')
		else:
			print "opcion incorrecta"
		time.sleep(5)

	# Funcion que prepara adecuadamente y ejecuta VLC para la emision de video
	# Parametros de entrada:
	# - tipo: tipo de video seleccionado (dvd, vcd o archivo)
	# - archivo: ruta del archivo de reproduccion, si no se ha elegido este tipo este paramentro contendra una cadena vacia
	# - calidad: calidad de la emision (baja, media, alta o real)
	# - formato: formato de la emision (normal, panoramico, cinemascope u original)
	def ejecutar_emision_video(self, tipo, archivo, calidad, formato):
		ip = self.interfaz_emision.get_ip()
		# Se determina la fuente de emision
		fuente = self.detectar_fuente(tipo, archivo)
		if fuente == '':
			return False
		if calidad == 'real': # Emiision a calidad real
			if formato == '0:0': # Formato original
				self.pid_proceso_emision_vlc = os.spawnlp(os.P_NOWAIT, '/usr/bin/vlc', '/usr/bin/vlc', '-q', '--quiet-synchro', '--width', '100', '--height', '200', '-vvv', '%s' % fuente, '--sout', '#duplicate{dst=display,dst=std{access=http,mux=ps,dst=%s:8080}}' % ip)
			else: # Formato modificado
				self.pid_proceso_emision_vlc = os.spawnlp(os.P_NOWAIT, '/usr/bin/vlc', '/usr/bin/vlc', '-q', '--quiet-synchro', '--width', '100', '--height', '200', '-vvv', '%s' % fuente, '--aspect-ratio', '%s' % formato,'--sout', '#duplicate{dst=display,dst=std{access=http,mux=ps,dst=%s:8080}}' % ip)
			time.sleep(5)
		else: # Emision con reduccion de calidad
			if calidad == 'baja':
				ratio = '512'
			elif calidad == 'media':
				ratio = '1024'
			elif calidad == 'alta':
				ratio = '2048'
			else:
				print "parametro incorrecto"
			# Se ejecutan los dos reproductores, uno para la emision y otro para la visualizacion (esto
			# solo se hace cuando la calidad no es real, debido a la recodificacion del video si se hace
			# en un solo reproductor el sistema se sobrecarga
			self.pid_proceso_emision_vlc = os.spawnlp(os.P_NOWAIT, '/usr/bin/vlc', '/usr/bin/vlc', '-q', '--quiet-synchro', '--reset-plugins-cache', '-vvv', '%s' % fuente, '--sout', '#transcode{vcodec=mp4v,vb=%s,scale=1}:duplicate{dst=standard{access=http,mux=ps,url=%s:8080}}' % (ratio, ip))
			time.sleep(10)
			if formato == '0:0':
				self.pid_proceso_emision_vlc_aux = os.spawnlp(os.P_NOWAIT, '/usr/bin/vlc', '/usr/bin/vlc', '--width', '100', '--height', '200', 'http://%s:8080' % ip)
			else:
				self.pid_proceso_emision_vlc_aux = os.spawnlp(os.P_NOWAIT, '/usr/bin/vlc', '/usr/bin/vlc', '--width', '100', '--height', '200', '--aspect-ratio', '%s' % formato, 'http://%s:8080' % ip)
	
	# Funcion que prepara adecuadamente y ejecuta VLC para la emision de audio
	# Parametros de entrada:
	# - tipo: tipo de emision de audio seleccionada (cd o archivo)
	# - archivo: ruta del archivo de reproduccion, si no se ha elegido este tipo este parametro contendra una cadena vacia
	def ejecutar_emision_audio(self, tipo, archivo):
		ip = self.interfaz_emision.get_ip()
		# Se determina la fuente de emision
		fuente = self.detectar_fuente(tipo, archivo)
		if fuente == '':
			return False
		
		# Ejecutar en segundo plano el reproductor con las opciones adecuadas
		if gtk.pygtk_version[0] >= 2 and gtk.pygtk_version[1] >= 10:
			self.pid_proceso_emision_vlc = os.spawnlp(os.P_NOWAIT, '/usr/bin/vlc', '/usr/bin/vlc', '-vvv', '%s' % fuente, '--sout', '#transcode{acodec=vorb,ab=128}:duplicate{dst=display,dst=standard{access=http,mux=ogg,url=%s:8080}}' % ip)
		else:
			self.pid_proceso_emision_vlc = os.spawnlp(os.P_NOWAIT, '/usr/bin/vlc', '/usr/bin/vlc', '-vvv', '%s' % fuente, '--sout', '#transcode{acodec=mpga,ab=128}:duplicate{dst=display,dst=standard{access=http,mux=raw,url=%s:8080}}' % ip)
		time.sleep(5)
	
	# Funcion asociada a los botones Emitir y Emitir a uno encargada de iniciar la emision de tipo seleccionado
	def emitir(self, widget, data = None):
		print self.interfaz_emision.get_ip()
		if self.interfaz_emision.get_ip() == 'IP no disponible':
			aviso = mensaje_aviso_crv.Mensaje_Aviso("No es posible emitir por el Cañón de Red Virtual, su equipo informático no se encuentra conectado a la red local. Revise la configuración de red de su ordenador y vuelva a intentar la emisión posteriormente.")
			return 1
		
		self.interfaz_emision.mostrar_en_emision()
		if self.tipo_emision == "escritorio": # Emision de escritorio
			# Opciones de la emision de escritorio
			calidad = self.control_crv_escritorio.get_calidad()
			bloqueo_clientes = self.control_crv_escritorio.get_bloqueo()
			if gtk.pygtk_version[0] >= 2 and gtk.pygtk_version[1] >= 10:
				self.icono_emision.mensaje_escritorio(calidad, bloqueo_clientes)

			# Ejecutar localmente la emision de escritorio
			if data != 'ind':
				self.ejecutar_emision_escritorio(calidad)
			time.sleep(3)
			if bloqueo_clientes == True:
				bloq = '0:0'
			else:
				bloq = '0:1'

			# Enviar el mensaje de emision a los receptores o a un receptor individual
			if data == 'ind':
				self.comenzar_emision_individual('opcion01_%s\0' % bloq)
			else:
				self.comenzar_emision('opcion01_%s\0' % bloq)
		elif self.tipo_emision == "video": # Emision de video
			# Opciones de la emision de video
			tipo_video = self.control_crv_video.get_tipo_video()
			archivo_video = self.control_crv_video.get_archivo()
			calidad = self.control_crv_video.get_calidad()
			formato = self.control_crv_video.get_formato()
			if gtk.pygtk_version[0] >= 2 and gtk.pygtk_version[1] >= 10:
				self.icono_emision.mensaje_video(tipo_video, archivo_video, calidad, formato)

			# Ejecutar localmente la emision de video
			if data != 'ind':
				if self.ejecutar_emision_video(tipo_video, archivo_video, calidad, formato) == False:
					self.detener(None)
					return False

			# Enviar el mensaje de emision a los receptores o a un receptor individual
			if data == 'ind':
				self.comenzar_emision_individual('opcion02_%s\0' % formato)
			else:
				self.comenzar_emision('opcion02_%s\0' % formato)
		elif self.tipo_emision == "audio": # Emision de audio
			# Opciones de la emision de audio
			tipo_audio = self.control_crv_audio.get_tipo_audio()
			archivo_audio = self.control_crv_audio.get_archivo()
			if gtk.pygtk_version[0] >= 2 and gtk.pygtk_version[1] >= 10:
				self.icono_emision.mensaje_audio(tipo_audio, archivo_audio)
		
			# Ejecutar localmente la emision de audio
			if data != 'ind':
				if self.ejecutar_emision_audio(tipo_audio, archivo_audio) == False:
					self.detener(None)
					return False		
	
			# Enviar el mensaje de emision a los receptores o a un receptor individual	
			if data == 'ind':
				self.comenzar_emision_individual('opcion05_000000\0')
			else:
				self.comenzar_emision('opcion05_000000\0')
		else:
			print "Opcion incorrecta"
		if gtk.pygtk_version[0] >= 2 and gtk.pygtk_version[1] >= 10:
			self.icono_emision.show()
		if data != 'ind':
			widget.set_sensitive(False)
			self.selector.set_sensitive(False)
			self.boton_emitir_receptor.set_sensitive(True)
			self.boton_parar.set_sensitive(True)
			self.boton_borrar_lista.set_sensitive(False)
			self.boton_borrar_seleccion.set_sensitive(False)
			self.boton_editar.set_sensitive(False)
		
	# Funcion que envia el mensaje de emision a un receptor individual
	# Paramentros de entrada:
	# - mensaje: cadena de caracteres que contrine el mensaje que se transmite, dependiendo
	#            del tipo de emision el mensaje sera uno u otro
	def comenzar_emision_individual(self, mensaje):
		# Se envia el mensaje al receptor seleccionado de la lista
		ip_cliente_sel = self.clientes.ip_fila_seleccionada()
		estado = self.clientes.estado_fila_seleccionada()
		if ip_cliente_sel != '' and estado != 'True':
			activador = comunicacion_crv.Emisor_Cliente()
			activador.emitir(ip_cliente_sel, mensaje)
			activador.terminar()

	# Funcion que envia a los clientes el mensaje de inicio de emision para que
	# se conecten automaticamente al emisor
	# Parametros de entrada:
	# - mensaje: cadena de caracteres que contiene el mensaje que se transmite, dependiendo
	#            del tipo de emison el mensaje sera uno u otro
	def comenzar_emision(self, mensaje):
		fichero_lista = open('%slista_clientes.txt' % self.PATHLISTA, 'r')
		lista_completa = fichero_lista.readlines()
		fichero_lista.close()
		for cliente in lista_completa: # a cada receptor conocido se manda el mensaje de emision
			datos_cliente = cliente.split(';;')
			pid = os.fork()
			if pid == 0:
				activador = comunicacion_crv.Emisor_Cliente()
				activador.emitir(datos_cliente[1], mensaje)
				activador.terminar()
				os._exit(0)
	
	# Funcion que envia a los clientes el mensaje de fin de emision
	# Solo es necesaria cuando se usa VLC (video y audio) para que el
	# receptor cierre el reproductor automaticamente cuando el emisor
	# detiene la emsion
	def terminar_emision(self):
		# Se crea un objeto para la comunicacion con los clientes
		activador = comunicacion_crv.Emisor_Cliente()
		fichero_lista = open('%slista_clientes.txt' % self.PATHLISTA, 'r')
		lista_completa = fichero_lista.readlines()
		fichero_lista.close()
		for cliente in lista_completa: # Para cada cliente se manda el mensaje de detener
			datos_cliente = cliente.split(';;')
			print datos_cliente[1]
			activador.emitir_cerrar(datos_cliente[1])
		activador.terminar()
	

	# Funcion asociada al boton Borrar y Borrar todo
	# En el caso de borrar: elimina el elemento seleccionado de la lista
	# En el caso de borrar todo: elimina la lista completa de receptores conocidos
        def borrar(self, widget, data = None):
		# Se muestra por pantalla una ventana de confirmacion
                confirmacion = gtk.Dialog("Confirmar")
                confirmacion.add_button("Cancelar", gtk.RESPONSE_CANCEL)
                confirmacion.add_button("Aceptar", gtk.RESPONSE_OK)
                if widget.get_label() == "Borrar todo": # Se ha elegido borrar la lista completa
			confirmacion.connect("response", self.clientes.borrar_lista, None)
			msg = "La lista de receptores conocidos sera eliminada completamente"
		elif widget.get_label() == "Borrar": # Se ha elegido borrar un elemento de la lista
			confirmacion.connect("response", self.clientes.borrar_cliente, None)
			msg = "El receptor seleccionado sera eliminado de la lista"
		else:
			print "Opcion incorrecta"

		mensaje = gtk.Label(msg)
		mensaje.set_justify(gtk.JUSTIFY_CENTER)
		confirmacion.vbox.pack_start(mensaje, True, True, 0)
		confirmacion.set_size_request(420, 150)
		mensaje.show()
		confirmacion.run()

	# Funcion asocioada al boton de Refrescar que actualiza la lista de receptores conocidos
	def refrescar(self, widget, data = None):
		print "ejecuta actual"
		self.clientes.get_lista_completa()
		print "ejecuta actual"
	
	# Funcion que detiene la emision por el CRV asociada al boton de Detener
	def detener(self, widget, data = None):
		# Se busca la emision en curso, si el pid correspondiente a VNC, VLC o VLC_AUX es
		# distinto de 0 significa que se encuentra en ejecucion y hay que terminar ese proceso
		if self.pid_proceso_emision_vnc != 0:
			os.kill(self.pid_proceso_emision_vnc, signal.SIGINT)
			self.pid_proceso_emision_vnc = 0
			self.terminar_emision()
		if self.pid_proceso_emision_vlc != 0:
			os.kill(self.pid_proceso_emision_vlc, signal.SIGKILL)
			self.pid_proceso_emision_vlc = 0
			self.terminar_emision()
		if self.pid_proceso_emision_vlc_aux != 0:
			os.kill(self.pid_proceso_emision_vlc_aux, signal.SIGKILL)
			self.pid_proceso_emision_vlc_aux = 0
			
		# Acciones adicionales para preparar la siguiente emision
		self.interfaz_emision.mostrar_en_crv_profesor()
		if gtk.pygtk_version[0] >= 2 and gtk.pygtk_version[1] >= 10:
			self.icono_emision.hide()
		self.clientes.todos_desconectados()
		self.selector.set_sensitive(True)
		self.clientes.get_lista_completa()
		self.boton_emitir.set_sensitive(True)
		self.boton_emitir_receptor.set_sensitive(False)
		self.boton_parar.set_sensitive(False)
		self.boton_borrar_lista.set_sensitive(True)
		self.boton_borrar_seleccion.set_sensitive(True)
		self.boton_editar.set_sensitive(True)

	# Funcion que corta las comunicaciones entre el emisor y los receptores
	def terminar(self):
		self.socket_entrante.cerrar()
	
	# Funcion que devuelve el pid del proceso de emision por VNC (Escritorio)
	def get_pid_proceso_emision_vnc(self):
		return self.pid_proceso_emision_vnc

	# Funcion que devuelve el pid del proceso de emsion por VLC (Video y audio)
	def get_pid_proceso_emision_vlc(self):
		return self.pid_proceso_emision_vlc

	# Funcion que devuelve el pid del proceso de emision por VLC auxiliar (Este proceso
	# solo se crea si se modifica la calidad de la emisio de video)
	def get_pid_proceso_emision_vlc_aux(self):
		return self.pid_proceso_emision_vlc_aux

	# Funcion para detectar la fuente de emision para la reproduccion de video y audio
	# Paramentros de entrada:
	# - tipo: tipo de emision que se va a realizar
	# - archivo: ruta del archivo que se va a reproducir en el caso de que se haya
	#            seleccionado la emision de archivo
	# Parametro de retorno:
	# -La funcion retorna la fuente que se utiliza para reproducir, es una cadena de texto
	#  que en el caso de no encontrarse la fuente apropiada se devolvera vacia
	def detectar_fuente(self, tipo, archivo):
		fuente = ''

		# Se buscan todos los lectores opticos que se encuentran instalados en el ordenador
		if os.access('/proc/sys/dev/cdrom/info', os.F_OK) == True:
			info_lectores_buffer = os.popen('cat /proc/sys/dev/cdrom/info', 'r')
			info_lectores = info_lectores_buffer.read()
			info_lectores = info_lectores.split('\n')
			for linea in info_lectores:
				pattern = re.compile('drive name')
				if re.search(pattern, linea) != None:
					dispositivos = linea.split('\t')
					break
		else:
			dispositivos = ''
		
		if tipo == 'dvd': # Emision de video desde el DVD
			for disp in dispositivos:
		        	if disp != '' and disp != 'drive name:':
					# Se comprueba que el dispositivo es un lector de DVD y que tiene un Disco de DVD de video introducido
			                info_buffer, salida, error_buffer = os.popen3('/usr/bin/dvd+rw-mediainfo /dev/%s' % disp, 'r')
			                error = error_buffer.read()
			                pattern_1 = re.compile('no media mounted')
			                pattern_2 = re.compile('non-DVD media mounted')
			                if re.search(pattern_1, error) == None and re.search(pattern_2, error) == None:
						fuente = "dvd:/dev/" + disp
						return fuente
			# En el caso de no encontrarse un medio adecuado se informa por pantalla
			aviso = mensaje_aviso_crv.Mensaje_Aviso("No es posible emitir un DVD por el Cañón de Red Virtual. Es probable que su equipo informático no disponga de unidad lectora de discos DVD o el disco introducido en el lector no se corresponda con un DVD de vídeo. En cualquier caso, revise su configuración y vuelva a intentar la emisión.")
		elif tipo == 'vcd': # Emision de video desde VCD
			for disp in dispositivos:
		        	if disp != '' and disp != 'drive name:':
					# Se comprueba que el disco introducido dentro del lector es un CD de audio correcto
					info_cd_buffer = os.popen('setcd -i /dev/%s' % disp, 'r')
					info_cd = info_cd_buffer.read()
					info_cd = info_cd.split('\n')
					pattern = re.compile('data disc')
					for linea in info_cd:
						if re.search(pattern, linea) != None:
							fuente = "vcd:/dev/" + disp
							return fuente
			# En el acso de no encontrarse un medio adecuado se informa por pantalla
			aviso = mensaje_aviso_crv.Mensaje_Aviso("No es posible emitir un VCD por el Cañón de Red Virtual. Es probable que su equipo informático no disponga de unidad lectora de CD o el disco introducido en el lector no se corresponde con un VCD. En cualquier caso, revise su configuración y vuelva a intentar la emisión.")
			
		elif tipo == 'cd': # Emision de audio desde CD
			for disp in dispositivos:
		        	if disp != '' and disp != 'drive name:':
					# Se comprueba que el disco introducido dentro del lector es un CD de audio correcto
					info_cd_buffer = os.popen('setcd -i /dev/%s' % disp, 'r')
					info_cd = info_cd_buffer.read()
					info_cd = info_cd.split('\n')
					pattern = re.compile('audio disc')
					for linea in info_cd:
						if re.search(pattern, linea) != None:
							fuente = "cdda:/dev/" + disp
							return fuente
			# En el acso de no encontrarse un medio adecuado se informa por pantalla
			aviso = mensaje_aviso_crv.Mensaje_Aviso("No es posible emitir un CD de audio por el Cañón de Red Virtual. Es probable que su equipo informático no disponga de unidad lectora de CD o el disco introducido en el lector no se corresponde con un CD de audio. En cualquier caso, revise su configuración y vuelva a intentar la emisión.")
		elif tipo == 'archivo': # Emision de video o audio desde un archivo
			if self.tipo_emision == 'video': # archivo de video
				nombre_archivo = self.control_crv_video.get_archivo()
			elif self.tipo_emision == 'audio': # archivo de audio
				nombre_archivo = self.control_crv_audio.get_archivo()
			else:	
				print "paramentro incorrecto"

			# Se comprueba que el archivo existe y es accesible por el usuario
			if os.access('%s' % nombre_archivo, os.F_OK) == True:
				return nombre_archivo
			else: # Se avisa por pantalla si el archivo seleccionado no existe o no se tiene acceso
				aviso = mensaje_aviso_crv.Mensaje_Aviso("No es posible reproducir el archivo indicado por el Cañón de Red Virtual. Es probable que no haya indicado correctamente el nombre o la ruta del archivo que desea reproducir o que el archivo no exista. En cualquier caso, seleccione correctamente el archivo que se debe reproducir e intente de nuevo la emisión.")
		else:
			print "parametro incorrecto"
		return fuente

	# Constructor de la clase
	# Se encarga de las siguientes operaciones:
	# - Creacion de la interfaz completa de la aplicacion de profesor:
	#	- Selector de interfaz de red
	#	- Selector de tipo de emision
	#	- Opciones de cada tipo de emision
	#	- Gestion de lista de receptores conocidos
	#	- Area de notificaciones
	def __init__(self):
		self.PATH = '/usr/local/crv/'
                self.PATHLISTA = '/usr/share/crv/'
		
		self.pid_proceso_emision_vnc = 0
		self.pid_proceso_emision_vlc = 0
		self.pid_proceso_emision_vlc_aux = 0
		self.tipo_emision = ''

		self.boxv = gtk.VBox(False, 10)
		self.total_fixed = gtk.Fixed()
		self.boxv.pack_start(self.total_fixed, True, True, 0)

		# Widget para la informacion del area de notificaciones de gnome
		if gtk.pygtk_version[0] >= 2 and gtk.pygtk_version[1] >= 10:
			self.icono_emision = Notificacion_Estado()

		# Seleccion de la interfaz de emision
		self.interfaz_emision = Gestor_Redes()
		self.total_fixed.put(self.interfaz_emision.interfaz_frame, 0, 0)
		self.interfaz_emision.show()

		# Seleccion del tipo de emision
		self.seleccion_frame = gtk.Frame()
		self.seleccion_panel = gtk.Fixed()
		self.seleccion_frame.set_size_request(410, 120)
		self.et_seleccion = gtk.Label()
		self.et_seleccion.set_markup("<b>Seleccionar el modo de funcionamiento del CRV</b>")
		self.et_seleccion.set_use_markup(True)
		self.seleccion_panel.put(self.et_seleccion, 10, 10)
		self.selector = gtk.RadioButton(None, "Emitir escritorio por el Cañón de Red Virtual")
		self.selector.connect("toggled", self.selector_emision, "1")
		self.tipo_emision = "escritorio"
		self.seleccion_panel.put(self.selector, 10, 30)
		self.selector.show()
		self.selector = gtk.RadioButton(self.selector, "Emitir vídeo por el Cañón de Red Virtual")
		self.selector.connect("toggled", self.selector_emision, "2")
		self.seleccion_panel.put(self.selector, 10, 50)
		self.emisor_activo="escritorio"
		self.selector.show()
		self.selector = gtk.RadioButton(self.selector, "Emitir audio por el Cañón de Red Virtual")
		self.selector.show()
		self.selector.connect("toggled", self.selector_emision, "3")
		self.seleccion_panel.put(self.selector, 10, 70)
		self.selector = gtk.RadioButton(self.selector, "Utilizar el Cañón de Red Virtual como alumno")
		self.selector.connect("toggled", self.selector_emision, "4")
		self.seleccion_panel.put(self.selector, 10, 90)
		self.seleccion_frame.add(self.seleccion_panel)
		self.total_fixed.put(self.seleccion_frame, 0, 90) 
		self.seleccion_frame.show()
		self.seleccion_panel.show()
		self.selector.show()
		self.et_seleccion.show()

		# Opciones del tipo de emision seleccionada
		# Se crean objetos para todos los tipos de emison y recepcion pero no son mostrados hasta que se seleccionan
		# por defecto se muestra el cotrol de escritorio
		self.opciones_frame = gtk.Frame()
		self.opciones_frame.set_label("Opciones de emisión de escritorio")
		self.opciones_fixed = gtk.Fixed()
		self.opciones_frame.set_size_request(410, 290)
		self.control_crv_video = Control_Crv_Video()
		self.opciones_fixed.put(self.control_crv_video.video_frame, 10, 10)
		self.control_crv_escritorio = Control_Crv_Escritorio()
		self.opciones_fixed.put(self.control_crv_escritorio.escritorio_frame, 10, 10)
		self.control_crv_audio = Control_Crv_Audio()
		self.opciones_fixed.put(self.control_crv_audio.audio_frame, 10, 10)
		self.crv_alumnos = crv_Alumnos()
		self.crv_alumnos.adaptar_crv_profesor()
		self.opciones_fixed.put(self.crv_alumnos.boxv, 10, -20)
		self.opciones_frame.add(self.opciones_fixed)
		self.total_fixed.put(self.opciones_frame, 0, 220)
		self.control_crv_escritorio.show()
		self.opciones_frame.show()
		self.opciones_fixed.show()

		# Lista de receptores conocidos
		self.clientes = list_cli_crv.Lista_Clientes()
		self.total_fixed.put(self.clientes.listado_fixed, 420, 0)
		self.clientes.todos_desconectados()
		self.clientes.show()

		# Botonera de emisiones
		self.botemi_frame = gtk.Frame()
		self.botemi_frame.set_shadow_type(gtk.SHADOW_NONE)
		self.botemi_fixed = gtk.Fixed()

		# Botonera de acciones listas
		self.botonera_gestor_lista = gtk.HButtonBox()
		self.botonera_gestor_lista.set_border_width(0)
		self.botemi_fixed.put(self.botonera_gestor_lista, -3, 15)
		self.botonera_gestor_lista.set_size_request(365, 10)
		self.botonera_gestor_lista.show()
		self.botonera_acciones = gtk.HButtonBox()
		self.botonera_acciones.set_border_width(0)
		self.botemi_fixed.put(self.botonera_acciones, -3, 50)
		self.botonera_acciones.set_size_request(365, 10)
		self.botonera_acciones.show()
		self.boton_borrar_lista = gtk.Button("Borrar todo")
		self.boton_borrar_lista.connect("clicked", self.borrar, None)
		self.boton_borrar_lista.set_size_request(1, 30)
		self.boton_borrar_lista.show()
		self.boton_borrar_seleccion = gtk.Button("Borrar")
		self.boton_borrar_seleccion.connect("clicked", self.borrar, None)
		self.boton_borrar_seleccion.set_size_request(60, 30)
		self.boton_borrar_seleccion.show()
		self.boton_editar = gtk.Button("Editar")
		self.boton_editar.set_size_request(60, 30)
		self.boton_editar.connect("clicked", self.clientes.editar_cliente, None)
		self.boton_editar.show()
		self.boton_refrescar = gtk.Button("Refrescar")
		self.boton_refrescar.connect("clicked", self.refrescar, None)
		self.boton_refrescar.set_size_request(60, 30)
		self.boton_refrescar.show()
		
		# Botonera de acciones de emison
		self.botonera_gestor_lista.add(self.boton_borrar_lista)
		self.botonera_gestor_lista.add(self.boton_borrar_seleccion)
		self.botonera_gestor_lista.add(self.boton_editar)
		self.botonera_gestor_lista.add(self.boton_refrescar)
		self.botonera_gestor_lista.set_homogeneous(False)
		self.boton_parar = gtk.Button("Detener Emisión")
		self.boton_parar.connect("clicked", self.detener, None)
		self.botonera_acciones.add(self.boton_parar)
		self.boton_parar.set_size_request(110, 30)
		self.boton_parar.set_sensitive(False)
		self.boton_parar.show()
		self.boton_emitir_receptor= gtk.Button("Emitir a uno")
		self.boton_emitir_receptor.connect("clicked", self.emitir, 'ind')
		self.botonera_acciones.add(self.boton_emitir_receptor)
		self.boton_emitir_receptor.set_sensitive(False)
		self.boton_emitir_receptor.show()
		self.boton_emitir = gtk.Button("Emitir a todos")
		self.boton_emitir.connect("clicked", self.emitir, None)
		self.botonera_acciones.add(self.boton_emitir)
		self.boton_emitir.show()
		self.botemi_frame.add(self.botemi_fixed)
		self.total_fixed.put(self.botemi_frame, 420, 440)
		self.botemi_frame.show()
		self.botemi_fixed.show()

		# Se crea el objeto encargado de gestionar las comunicaciones con los receptores
		self.socket_entrante = comunicacion_crv.Recepcion_Clientes()

		# Dibujar todos los objetos en la ventana de la aplicacion	
		self.total_fixed.show()	
		self.boxv.show()

# Clase para mostrar la interfaz del CRV para alumnos (recepcion)
class crv_Alumnos:
	# Funcion para mostrar el bloque del CRV Alumnos
	def show(self):
		self.boxv.show()

	# Funcion para ocultar el bloque del CRV Alumnos
	def hide(self):
		self.boxv.hide()

	# Funcion para conectarse con el emisor del CRV
	def conectar_emisor(self, widget, data = None):	
		print "conectar con el servidor"

		# Se comprueba que la direccion IP introducida es correcta
		if self.in_ip_3.get_text() != '' and self.in_ip_4.get_text() != '':
			in_ip_1_num = string.atoi(self.in_ip_1.get_text())
			in_ip_2_num = string.atoi(self.in_ip_2.get_text())
			in_ip_3_num = string.atoi(self.in_ip_3.get_text())
			in_ip_4_num = string.atoi(self.in_ip_4.get_text())
			if in_ip_3_num >= 0 and in_ip_3_num <= 255 and in_ip_4_num >= 0 and in_ip_4_num <= 255 and in_ip_1_num == 192 and in_ip_2_num == 168:
				# Reconstruccion de IP y se intenta conectar con el emisor
				ip = self.in_ip_1.get_text() + "." + self.in_ip_2.get_text() + "." + self.in_ip_3.get_text() + "." + self.in_ip_4.get_text()
				aviso_emisor = comunicacion_crv.Comunicador_Clientes()
				if aviso_emisor.emitir(ip) == 0: # Si el emisor responde se conecta a el para recepcionar
					conexion = CrvAl(self.tipo_emi, ip)
					#os._exit(0)
				else: # Si la conexion no es posible se avisa por pantalla
					aviso = mensaje_aviso_crv.Mensaje_Aviso("No ha sido posible conectar con el emisor del Cañón de Red Virtual. Es posible que la dirección IP introducida no se corresponda con un emisor, el emisor haya concluido la emisión o que este ordenador no se encuentre conectado a la red. En cualquier caso, revise su configuración e intente de nuevo conectar con el emisor.")
					print "error de conexion"
		# Si la direccion IP no es correcta se avisa por pantalla
			else:
				aviso = mensaje_aviso_crv.Mensaje_Aviso("La dirección IP introducida no es correcta, por favor revise los datos introducidos y vuelva a intentar conectar de nuevo con el emisor.")
		else:
				aviso = mensaje_aviso_crv.Mensaje_Aviso("La dirección IP introducida no es correcta, por favor revise los datos introducidos y vuelva a intentar conectar de nuevo con el emisor.")

	# Funcion asociada al boton Cancelar del receptor
	def cancelar(self, widget, data = None):
		os._exit(1)

	# Funcion para determinar la seleccion de recepcion realizada por el
	# usuario	
	def selector_visualizacion(self, widget, data = None):
		if data == '1':
			if widget.get_active():
				self.tipo_emi = 'opcion01_0:0'
		elif data == '2':
			if widget.get_active():
				self.tipo_emi = 'opcion02_0:0'
		elif data == '3':
			if widget.get_active():
				self.tipo_emi = 'opcion05_000000'
		else:
			print "Opcion incorrecta"

	# Funcion que adapta los componentes del CRV Alumnos para intregrarse
	# dentro del CRV Profesor cuando se selecciona la opcion de funcionar
	# como alumno
	def adaptar_crv_profesor(self):
		self.sel_frame.set_shadow_type(gtk.SHADOW_NONE)
		self.sel_frame.set_label("")
		self.dirip_frame.set_shadow_type(gtk.SHADOW_NONE)
		self.dirip_frame.set_label("")
		self.total_fixed.move(self.dirip_frame, 0, 120)
		self.total_fixed.put(self.botonera_frame, 0, 240)

	# Constructor de la clase
	# Crea todos los elementos que se visualiza en la interfaz y le incluye la funcionalidad
	def __init__(self):
		self.tipo_emi = 'opcion01_0:0'  # Tipo de recepcion por defecto

		self.boxv = gtk.VBox(False, 10)
		self.total_fixed = gtk.Fixed()
		self.boxv.pack_start(self.total_fixed, True, True, 0)
	
		#Seleccion del tipo de recepcion de los los alumnos	
		self.sel_frame = gtk.Frame("Seleccionar Recepción")
		self.sel_panel = gtk.Fixed()
		self.et_selector = gtk.Label()
		self.et_selector.set_markup("<b>Seleccionar que se desea visualizar desde el \nCañón de Red Virtual</b>")
		self.et_selector.set_use_markup(True)
		self.sel_panel.put(self.et_selector, 10, 10)
		self.selector = gtk.RadioButton (None, "Visualizar escritorio desde el Cañón de Red Virtual")
		self.sel_panel.put(self.selector, 10, 50)
		self.selector.connect("toggled", self.selector_visualizacion, "1")
		self.selector.show()
		self.selector = gtk.RadioButton (self.selector, "Visualizar vídeo desde el Cañón de Red Virtual")
		self.sel_panel.put(self.selector, 10, 70)
		self.selector.connect("toggled", self.selector_visualizacion, "2")
		self.selector.show()
		self.selector = gtk.RadioButton (self.selector, "Escuchar audio desde el Cañón de Red Virtual")
		self.sel_panel.put(self.selector, 10, 90)
		self.selector.connect("toggled", self.selector_visualizacion, "3")
		self.selector.show()
		self.sel_frame.add(self.sel_panel)
		self.sel_frame.set_size_request(390, 140)
		self.sel_frame.show()
		self.sel_panel.show()
		self.et_selector.show()
		self.sel_panel.show()
		self.total_fixed.put(self.sel_frame, 0, 0)

		#Cuadro para introducir la direccion IP de emision
		self.dirip_frame = gtk.Frame("Conexión")
		self.dirip_panel = gtk.Fixed()
		self.dirip_frame.set_size_request(390, 110)
		self.et_ip = gtk.Label()
		self.et_ip.set_markup("<b>Introduzca la dirección IP del ordenador que está \nemitiendo por el Cañón de Red Virtual</b>")
		self.et_ip.set_use_markup(True)
		self.dirip_panel.put(self.et_ip, 10, 10)

		self.dirip_frame.show()
		self.dirip_panel.show()
		self.et_ip.show()
		
		# Se crean 4 widgets de entrada de texto, uno por cada octeto de la direccion IP
		# Se prefijan los dos primeros octetos a 192 y 168 respectivamente para facilitar su uso
		# estos parametros prefijados se pueden modificar
		# Este formato es mas adecuado para comprobar si la direccion IP es valida
		self.in_ip_1 = gtk.Entry(3)
		self.in_ip_1.set_size_request(40, 24)
		self.in_ip_1.set_alignment(1.0)
		self.in_ip_1.set_text("192")
		self.et_ip_1 = gtk.Label(".")
		self.dirip_panel.put(self.et_ip_1, 53, 68)
		self.dirip_panel.put(self.in_ip_1, 10, 60)

		self.in_ip_2 = gtk.Entry(3)
		self.in_ip_2.set_size_request(40, 24)
		self.in_ip_2.set_alignment(1.0)
		self.in_ip_2.set_text("168")
		self.et_ip_2 = gtk.Label(".")
		self.dirip_panel.put(self.et_ip_2, 103, 68)
		self.dirip_panel.put(self.in_ip_2, 60, 60)

		self.in_ip_3 = gtk.Entry(3)
		self.in_ip_3.set_size_request(40, 24)
		self.in_ip_3.set_alignment(1.0)
		self.et_ip_3 = gtk.Label(".")
		self.dirip_panel.put(self.et_ip_3, 153, 68)
		self.dirip_panel.put(self.in_ip_3, 110, 60)

		self.in_ip_4 = gtk.Entry(3)
		self.in_ip_4.set_size_request(40, 24)
		self.in_ip_4.set_alignment(1.0)
		self.dirip_panel.put(self.in_ip_4, 160, 60)

		self.in_ip_1.show()
		self.et_ip_1.show()
		self.in_ip_2.show()
		self.et_ip_2.show()
		self.in_ip_3.show()
		self.et_ip_3.show()
		self.in_ip_4.show()

		self.dirip_frame.add(self.dirip_panel)
		self.total_fixed.put(self.dirip_frame, 0, 150)
		

		# Botonera de acciones
		# Se incluyen dos botones
		# - Aceptar para conectar con el emisor
		# - Cancelar para terminar la ejecucion de la aplicacion y no conectar
		self.botonera_frame = gtk.Frame()
		self.botonera_panel = gtk.Fixed()
		self.botonera_frame.set_size_request(390, 50)
		self.botonera_frame.set_shadow_type(gtk.SHADOW_NONE)
		self.conectar = gtk.Button("Conectar")
		self.conectar.connect("clicked", self.conectar_emisor, None)
		self.cancelar_conexion = gtk.Button("Cancelar")
		self.cancelar_conexion.connect("clicked", self.cancelar, None)
		self.botonera_panel.put(self.conectar, 290, 10)
		self.botonera_panel.put(self.cancelar_conexion, 180, 10)
		self.conectar.set_size_request(100, 30)
		self.cancelar_conexion.set_size_request(100, 30)
		self.botonera_frame.add(self.botonera_panel)

		# Dibujar todos los elementos en la ventana
		self.conectar.show()
		self.cancelar_conexion.show()
		self.botonera_panel.show()
		self.botonera_frame.show()
		self.total_fixed.put(self.botonera_frame, 0, 270)
		self.total_fixed.show()

# Clase para la ejecucion del CRV por parte de alumno desde el demonio crvcliented
# Se encarga de ejecutar los comandos necesarios para conectar con la emision automaticamente
class CrvAl:
	# Constructor de la clase
	# Paramentros de entrada:
	# - opcion : opcion enviada por el emisor para distinguir la aplicacion a la que se debe conectar
	#	Valores psobles:
	#	- opcion01_0:0   Conectar a escritorio con bloqueo de pantalla
	#	- opcion01_0:1   Conectar a escritorio sin bloqueo de pantalla
	#	- opcion02_XXX:XXX  Conectar con video donde XXX:XXX indica el ratio
	#	- opcion05_000000 Conectar con audio
	def __init__(self, opcion, ip):
		opcion_aux = opcion.split("_")
		ratio = opcion_aux[1]
		opcion = opcion_aux[0]

		# Se distingue que tipo de emision se esta realizando
		if opcion == "opcion01":
			if ratio == "0:1": # Escritorio sin bloqueo de ventana
				os.spawnlp(os.P_NOWAIT, '/usr/bin/xvnc4viewer', '/usr/bin/xvnc4viewer', '-MenuKey=""', '-UseLocalCursor=0', '%s:50000' % ip)
			elif ratio == "0:0": # Escritorio con bloqueo de ventana
				os.spawnlp(os.P_NOWAIT, '/usr/bin/xvnc4viewer', '/usr/bin/xvnc4viewer', '-fullscreen', '-MenuKey=""', '-UseLocalCursor=0', '%s:50000' % ip)
			else:
				print "Ratio incorrecto"

		elif opcion == "opcion02":
			if ratio == "0:0": # Video sin modificar ratio
				pid_proceso_recepcion = os.spawnlp(os.P_NOWAIT, '/usr/bin/vlc', '/usr/bin/vlc', '--http-caching', '10000', '--loop', 'http://%s:8080' % ip)
			else: # Video con ratio modificado
				pid_proceso_recepcion = os.spawnlp(os.P_NOWAIT, '/usr/bin/vlc', '/usr/bin/vlc', '--aspect-ratio', '%s' % ratio, '--http-caching', '10000', '--loop', 'http://%s:8080' % ip)
		elif opcion == "opcion05": #Reproduccion de audio
			pid_proceso_recepcion = os.spawnlp(os.P_NOWAIT, '/usr/bin/vlc', '/usr/bin/vlc', '--loop', 'http://%s:8080' % ip)
		elif opcion == "opcionXX":
			os.spawnlp(os.P_NOWAIT, '/usr/bin/killall', '/usr/bin/killall', 'vlc')
			os.spawnlp(os.P_NOWAIT, '/usr/bin/killall', '/usr/bin/killall', 'xvnc4viewer')
			os.spawnlp(os.P_NOWAIT, '/usr/bin/killall', '/usr/bin/killall', 'xscreensaver')
		else:
			print "Opcion incorrecta"
			os._exit(1)
		
# Clase principal de la aplicacion principal	
class Crv:
	def delete_event(self, widget, data = None):
		if self.es_Profesor() == True:
			self.crv.terminar()
			self.crv.detener(widget)
		gtk.main_quit()

	def destroy(self, widget, data = None):
		if self.es_Profesor() == True:
			self.crv.terminar()
			self.crv.detener(widget)
		gtk.main_quit()

	# Funcion para comprobar que el usuarios ejecutor pertenece al grupo
	# profesores
	# Devuelve:
	#  - True si el usuario pertenece al grupo profesores
	#  - False si el usuarios no pertenece al grupo profesores
	def es_Profesor(self):
		gid = os.getgid()
		if self.existe_grupo_profesores() == True:
			prof = grp.getgrnam('profesores')
			if prof[2] == gid:
				return True
			else:	
				return False

	# Funcion para comprobar que el grupo de profesores esta creado
	# Devuelve:
	#  - True si el grupo profesores esta creado
	#  - False si el grupo profesores no esta creado
	def existe_grupo_profesores(self):
		grupos = grp.getgrall()
		for elemento in grupos:
			if elemento[0] == 'profesores':
				return True
		return False

	# Constructor de la clase CRV
	# Dependiendo de los privilegios del usuario que ejecuta la aplicacion
	# se mostrara la interfaz de profesor o de alumno
	def __init__(self):
		self.PATH = '/usr/local/crv/'

		#Creacion de la ventana principal de la aplicacion, generica
		self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.window.set_border_width(10)
		self.window.set_icon_from_file("%simg/crv_icon.png" % self.PATH)
		self.window.connect("delete_event", self.delete_event)
		self.window.connect("destroy", self.destroy)
		self.window.set_resizable(False)
		self.box = gtk.VBox(False, 30)
		self.window.add(self.box)

		# Diferenciar entre profesores y el resto de usuarios
		if self.es_Profesor() == True:
			self.window.set_title("CRV - Cañón de Red Virtual - Emisor")
			self.crv = crv_Profesor()
			self.box.pack_start(self.crv.boxv, True, True, 0)
			self.window.set_size_request(800, 530)
		else:	
			self.window.set_title("CRV - Cañón de Red Virtual - Receptor")
			self.crv = crv_Alumnos()
			self.box.pack_start(self.crv.boxv, True, True, 0)
			self.crv.show()
			self.window.set_size_request(410,340)
			
		self.box.show()
		self.window.show()
